#include "gamegroup.h"
#include "application.h"
#include "physicsmgr.h"
#include "scenemgr.h"
#include "sprite.h"
#include "gameobjectlink.h"
#include <assert.h>
#include "physicsobject.h"

using namespace acUtility;

CGameGroup::CGameGroup()
{
	anchor       = 0;
	anchorOffset = vector2(0,0);
	position     = vector2(0,0);
}

CGameGroup::~CGameGroup()
{
	for( unsigned int n = 0; n < blockers.size(); n++ )
		app->physicsMgr->DestroyObject(blockers[n]);
	blockers.clear();

	for( unsigned int n = 0; n < sprites.size(); n++ )
	{
		app->sceneMgr->RemoveSprite(sprites[n]);
		delete sprites[n];
	}
	sprites.clear();

	for( unsigned int n = 0; n < objects.size(); n++ )
		objects[n]->Release();
	objects.clear();

	if( anchor )
		anchor->Release();
}

void CGameGroup::SynchSceneAndPhysics()
{
	// Move the sprites to the new position according to the physics objects
	if( blockers.size() )
		position = blockers[0]->GetPosition() - blockerOffsets[0];
	else if( objects.size() )
		position = objects[0]->GetPosition() - objectOffsets[0];

	for( unsigned int n = 0; n < sprites.size(); n++ )
		sprites[n]->pos = position + spriteOffsets[n];
}

void CGameGroup::SetPosition(const vector2 &pos)
{
	position = pos - anchorOffset;

	// Update all the blockers, sprites, and objects
	for( unsigned int n = 0; n < blockers.size(); n++ )
		blockers[n]->SetPosition(position + blockerOffsets[n]);

	for( unsigned int n = 0; n < sprites.size(); n++ )
		sprites[n]->pos = position + spriteOffsets[n];

	for( unsigned int n = 0; n < objects.size(); n++ )
		objects[n]->SetPosition(position + objectOffsets[n]);
}

vector2 CGameGroup::GetPosition()
{
	return position + anchorOffset;
}

void CGameGroup::SetVelocity(const vector2 &vel)
{
	// Set the velocity for all blockers
	for( unsigned int n = 0; n < blockers.size(); n++ )
		blockers[n]->SetVelocity(vel);
	// and objects
	for( unsigned int n = 0; n < objects.size(); n++ )
		objects[n]->SetVelocity(vel);
}

vector2 CGameGroup::GetVelocity()
{
	// Get the velocity from one of the blockers or objects
	if( blockers.size() )
		return blockers[0]->GetVelocity();
	if( objects.size() )
		return objects[0]->GetVelocity();

	return vector2(0,0);
}

void CGameGroup::SetAnchor(CGameObjectLink *a)
{
	if( anchor )
		anchor->Release();

	anchor = a;
	if( anchor )
	{
		anchor->AddRef();

		// Set relative position
		anchorOffset = a->GetPosition();
	}
}

void CGameGroup::AddSprite(CSprite *spr)
{
	sprites.push_back(spr);
	
	// Set relative position
	spriteOffsets.push_back(spr->pos);
}

void CGameGroup::AddBlocker(CPhysicsObject *obj)
{
	blockers.push_back(obj);

	// Set relative position
	blockerOffsets.push_back(obj->GetPosition());
}

void CGameGroup::AddObject(CGameObjectLink *obj)
{
	objects.push_back(obj);
	obj->AddRef();

	// Set relative position
	objectOffsets.push_back(obj->GetPosition());
}