#ifndef GAMEMGR_H
#define GAMEMGR_H

#include <string>
#include <vector>
#include <map>
#include "acutil_rand.h"
#include "acutil_math.h"
#include "add_on\scripthandle\scripthandle.h"


namespace acGraphics
{
	class CGraphics;
}

namespace acUtility
{
	class CConfig;
}

class CGameObject;
class CGameObjectLink;
class CGameObjectFactory;
class CSceneMgr;
class CGameGroup;

class CGameMgr
{
public:
	CGameMgr();
	~CGameMgr();

	int  Init(acUtility::CConfig &cfg);
	void Frame();
	float GetFrameDeltaTime();
	float GetRandomNumber();
	CGameObjectLink *FindObjectByName(const std::string &name);
	CGameObjectLink *SpawnObject(const std::string &type, const acUtility::vector2 &pos, const acUtility::vector2 &velocity);
    void SetTimeMultiplier(float multipler);
	void DestroyObject(CGameObject *obj);
	void QueueObjectKill(CGameObjectLink *link);
	float RayCast(const CGameObjectLink *ignoreObj, int query, const acUtility::vector2 &start, const acUtility::vector2 &end, const CGameObjectLink **firstObj, acUtility::vector2 *hitVelocity);
	void QueueTouch(CGameObjectLink *origin, const CGameObjectLink *other);
	void SetCameraPosition(const acUtility::vector2 &pos);
	acUtility::vector2 GetCameraPosition();
	acUtility::vector2 ScreenToWorld(const acUtility::vector2 &pos);
	CGameObject *AddObject(const std::string &type, const acUtility::vector2 &pos, int zOrder, const std::vector<std::string> &nameValuePairs, CGameGroup *group);
	void AddGroup(CGameGroup *grp);
	int  GenerateLevel(const std::string &logic);
	int  LoadLevel(const std::string &level);
	void EndLevel(int code);
	void RestartLevel();
	int  StartGame();
	void PauseGame();
	void ResumeGame();
	void EndGame();
	void QuitGame();
	bool GetProperty(const std::string &prop, __int64 &out);
	bool SetProperty(const std::string &prop, __int64 val);
	bool IsGameOn();
	void ShowMenu(unsigned int id);
	void SendMessage(const CGameObjectLink *to, unsigned int message, CScriptHandle arg);
	acUtility::vector2 GetNearestPathNode(const acUtility::vector2 &from, const acUtility::vector2 &to);
	void SetGravity(const acUtility::vector2 &grav);
	void FindObjectsWithinRange(const acUtility::vector2 &center, float radius, std::vector<CGameObject*> &objects);

	// This flag will be true while the game is actually running
	bool isGameOn;
	bool isInMenu;

	// Used by recorder
	unsigned int GetRandomSeed();
	void         SetRandomSeed(unsigned int);

protected:
	friend class CConsole;

	void InitLevel();

	CGameObject *CreateObject(const char *type, const acUtility::vector2 &pos);

	std::vector<CGameObjectLink*>        objectsToKill;
	std::vector<CGameObjectLink*>        objectsToTouch;

	struct SObjMsg
	{
		CGameObjectLink *to;
		unsigned int     msgId;
		CScriptHandle    arg;
	};
	std::vector<SObjMsg> objectMessages;


	std::vector<CGameObjectFactory*>     objectFactories;
	std::vector<CGameObject*>            objects;
	std::vector<CGameGroup*>             groups;
	acUtility::CSimpleRandomizer         randomizer;
	float                                timeMultiplier;

	bool                                 callEndLevel;
	int                                  endLevelCode;

	std::string                          gameScript;

	// Level properties
	std::map<std::string, __int64>       levelProperties;

	// This is the frame delta time
	float                                frameDeltaTime;
};

#endif