#ifndef GAMEOBJECT_H
#define GAMEOBJECT_H

#include <string>
#include "acutil_math.h"
#include <angelscript.h>
#include <vector>

class CSprite;
class CSpriteAnimation;
class CGameObjectLink;
class CGameObjectFactory;
class CPhysicsObject;
class CGameGroup;

class CGameObject
{
public:
	CGameObject(CGameObjectFactory *factory);
	~CGameObject();

	void Frame();

	void SetPosition(const acUtility::vector2 &pos);
	acUtility::vector2 GetPosition() const;
	void SetVelocity(const acUtility::vector2 &v);
	acUtility::vector2 GetVelocity() const;
	void MoveTo(const acUtility::vector2 &pos, float speed);
	void SetAnimation(const std::string &anim);
	void SetRotation(float rot);
	float GetRotation() const;
	// TODO: Should have a RotateTo(float angle, float speed);

	void OnCollide(const CGameObjectLink *obj, const acUtility::vector2 &v, const acUtility::vector2 &n, float approachSpeed);
	void OnProximity(const CGameObjectLink *obj, bool isNearing);

	bool GetProperty(const std::string &prop, __int64 &out);

	void SynchSceneAndPhysics();

	void  SetTimer(float time, int id);
	float GetTimer(int id) const;
	void  KillTimer(int id);

	void SetTransparency(float fraction);

	void Kill();
	void SetGravity(const acUtility::vector2 &g);
	void ClearGravity();
	void SetCollision(bool collision);
	const std::string &GetType() const;
	void Touch(const CGameObjectLink *other);

	void PlaySound(const std::string &snd, float volume);

	CGameObjectFactory            *m_factory;
	std::string                    m_name;
	CGameObjectLink               *m_link;
	CSprite                       *m_sprite;
	asIScriptObject               *m_scriptObj;

	// Controlled movement
	acUtility::vector2             m_targetPos;
	float                          m_moveSpeed; // If > 0 then we're moving

	// Animation
	CSpriteAnimation              *m_animation;
	float                          m_timeUntilNextFrame;
	unsigned int                   m_currentFrame;


	CPhysicsObject                *m_physicsObject;
	bool                           m_awake;
	CGameGroup                    *m_group;

	struct STimer
	{
		float time;
		int   id;
	};
	std::vector<STimer>            m_timers;

protected:
};

#endif
