#ifndef GAMEOBJECTLINK_H
#define GAMEOBJECTLINK_H

// 
// This object is a reference counted link to the game object. It allows the 
// game manager to sever all references to the game object without having to keep 
// track of the individual references themselves. This avoids the need for 
// a more complex routine for resolving circular references between game objects
// and script objects.
//

#include <string>
#include "acutil_math.h"
#include "gameobject.h"

class CGameObject;

#define IMPL_FUNC(func)               { if( obj ) obj->func; }
#define IMPL_FUNC_RET(func, ret)      { if( obj ) return obj->func; return ret; }
#define IMPL_FUNC_RET_REF(func, type) { if( obj ) return obj->func; static type ret; return ret; }
#define IMPL_SET(prop)                { if( obj ) obj->prop; }
#define IMPL_GET_REF(prop, type)      { if( obj ) return obj->prop; static type ret; return ret; }
#define IMPL_GET(prop, def)           { if( obj ) return obj->prop; return def; }

class CGameObjectLink
{
public:
	CGameObjectLink(CGameObject *obj);
	void AddRef() const;
	void Release() const;

	acUtility::vector2 GetPosition() const                        IMPL_FUNC_RET(GetPosition(), acUtility::vector2(0,0));
	void SetPosition(const acUtility::vector2 &p)                 IMPL_FUNC(SetPosition(p));
	acUtility::vector2 GetVelocity() const                        IMPL_FUNC_RET(GetVelocity(), acUtility::vector2(0,0));
	void SetVelocity(const acUtility::vector2 &v)                 IMPL_FUNC(SetVelocity(v));
	const std::string &GetName() const                            IMPL_GET_REF(m_name, std::string);
	void SetName(const std::string &n)                            IMPL_SET(m_name = n);
	bool GetAwake() const                                         IMPL_GET(m_awake, false);
	void SetAwake(bool b)                                         IMPL_SET(m_awake = b);
	float GetRotation() const                                     IMPL_FUNC_RET(GetRotation(), 0);
	void SetRotation(float r)                                     IMPL_FUNC(SetRotation(r));
	bool GetProperty(const std::string &prop, __int64 &out) const IMPL_FUNC_RET(GetProperty(prop, out), false);
	void SetAnimation(const std::string &anim)                    IMPL_FUNC(SetAnimation(anim));
	void SetCollision(bool collision)                             IMPL_FUNC(SetCollision(collision));
	void SetGravity(const acUtility::vector2 &g)                  IMPL_FUNC(SetGravity(g));
	void ClearGravity()                                           IMPL_FUNC(ClearGravity());
	const std::string &GetType() const                            IMPL_FUNC_RET_REF(GetType(), std::string);
	void SetTimer(float time, int id)                             IMPL_FUNC(SetTimer(time,id));
	float GetTimer(int id) const                                  IMPL_FUNC_RET(GetTimer(id), 0);
	void KillTimer(int id)                                        IMPL_FUNC(KillTimer(id));
	void Touch(const CGameObjectLink *other)                      IMPL_FUNC(Touch(other));
	void Kill()                                                   IMPL_FUNC(Kill());
	void SetTransparency(float fraction)                          IMPL_FUNC(SetTransparency(fraction));
	void PlaySound(const std::string &snd, float vol)             IMPL_FUNC(PlaySound(snd, vol));
	void MoveTo(const acUtility::vector2 &pos, float speed)       IMPL_FUNC(MoveTo(pos, speed));

    CGameObject *obj;

protected:
	~CGameObjectLink() {}
	mutable int refCount;
};

#endif