#ifndef GUIELEMENT_H
#define GUIELEMENT_H

#include <string>
#include <vector>
#include "acutil_math.h"

// TODO: It should be possible to establish a hierarchy of CGuiElements. It must not be allowed to create cycles

class asIScriptFunction;

class CGuiElement
{
public:
	CGuiElement(const acUtility::vector2 &pos, const acUtility::vector2 &size);
	virtual void AddRef();
	virtual void Release();

	virtual void AddChild(CGuiElement *child);
	virtual void RemoveChild(CGuiElement *child);

	virtual void Frame();
	virtual void Render(const acUtility::matrix4x4 &parentMtx);

	virtual bool HandleClick();
	virtual bool HandleMouseOver();

	virtual bool IsMouseOver();

	virtual bool IsChild(CGuiElement *el);
	virtual void SetTimer(float time, int id);
	virtual void KillTimer(int id);

	// Window
	acUtility::vector2 position;  // absolute position
	acUtility::vector2 size;      // absolute size
	acUtility::vector2 center;    // relative center
	float              rotation;  // rotation around center
	acUtility::vector2 scale;     // scale around center
	int                zPosition; // z order, less is closer to camera

	// Texture
	acUtility::vector2 srcBottomLeft;
	acUtility::vector2 srcTopRight;
	int                srcImageIndex;

	// Text
	std::string        text;
	int                fontIndex;
	int                align;
	unsigned int       color;

	// Event handlers
	asIScriptFunction *onClickHandler;
	asIScriptFunction *onMouseOverHandler;
	asIScriptFunction *onTimerHandler;

	// Timers
	struct STimer
	{
		STimer(float t, int i) : time(t), id(i) {}
		float time;
		int   id;
	};
	std::vector<STimer> timers;

	// Relationship
	CGuiElement               *parent;
	std::vector<CGuiElement*>  children;

protected:
	virtual ~CGuiElement();
	virtual void DrawSelf();

	int         refCount;
};

#endif