#ifndef GUIMGR_H
#define GUIMGR_H

#include <vector>
#include <string>
#include "acutil_math.h"

class CGuiElement;
class CGuiListBox;
namespace acUtility
{
	class CConfig;
}

namespace acGraphics
{
	class CGraphicsBuffer;
	class CFont;
}

class CGuiMgr
{
public:
	CGuiMgr();
	~CGuiMgr();

	int  Init(const acUtility::CConfig &config);
	void Uninit();
	void Frame();
	void Render();
	void SetReferenceSize(int width, int height);
	int  LoadScript(const std::string &script);

	bool GetWantInput() const;
	void SetWantInput(bool val);

	CGuiElement *CreateElement(const acUtility::vector2 &pos, const acUtility::vector2 &size, CGuiElement *parent);
	CGuiListBox *CreateListBox(const acUtility::vector2 &pos, const acUtility::vector2 &size, CGuiElement *parent);
	void RemoveElement(CGuiElement *el);
	void SetMouseElement(CGuiElement *el);
	acUtility::vector2 GetMousePos();
	acUtility::vector2 GetNormalizedMousePos();
	acUtility::vector2 NormalizePosition(const acUtility::vector2 &pos) const;

	int LoadImage(const std::string &name);
	int LoadFont(const std::string &name);

	void DrawRect(const acUtility::vector2 &bottomLeft, const acUtility::vector2 &topRight, int srcImage, const acUtility::vector2 &srcBottomLeft, const acUtility::vector2 &srcTopRight);
	void DrawText(const acUtility::vector2 &pos, const std::string &text, int font, int align, unsigned int color);
	void GetTextRect(acUtility::vector2 &ll, acUtility::vector2 &tr, const std::string &text, int font);

	acGraphics::CFont *GetFont(int index);

	void RemoveAll();

	int defaultFontIndex;

protected:
	friend class CGuiElement;

	CGuiElement *ElementExists(CGuiElement *el);

	int                              referenceWidth;
	int                              referenceHeight;
	int                              effectId;
	acUtility::vector2               mousePos;
	CGuiElement                     *currentMouseOver;
	bool                             hasScript;

	CGuiElement                               *mouseElement;
	std::vector<CGuiElement *>                 topLevelWindows;
	std::vector<acGraphics::CGraphicsBuffer *> images;
	std::vector<acGraphics::CFont *>           fonts;
};

#endif