#include "pathedge.h"
#include <assert.h>

using namespace acUtility;

CPathEdge::CPathEdge(vector2 &v1, vector2 &v2, CPathNode *o)
{
	p1 = v1;
	p2 = v2;
	next = 0;
	prev = 0;
	wall = true;
	owner = o;
	neighbour = 0;
}

CPathEdge *CPathEdge::Split(vector2 &v)
{
	CPathEdge *ne = new CPathEdge(v, p2, owner);
	ne->wall = wall;
	ne->prev = this;
	ne->next = next;
	if( next )
		next->prev = ne;
	next = ne;
	p2 = v;
	return ne;
}

void CPathEdge::SetNext(CPathEdge *e)
{
	assert( p2 == e->p1 );
	assert( owner == e->owner );
	assert( next == 0 && e->prev == 0 );
	
	next = e;
	e->prev = this;
}

void CPathEdge::SetNeighbour(CPathEdge *n)
{
	assert( p1 == n->p2 && p2 == n->p1 );
	assert( n->neighbour == 0 && neighbour == 0 );
	assert( !wall && !n->wall );

	neighbour = n;
	n->neighbour = this;
}

vector2 CPathEdge::GetNormal()
{
	vector2 nrm = p2 - p1;
	nrm.normalize();
	return vector2(-nrm.y, nrm.x);
}

bool CPathEdge::FindWallCornerAfter(vector2 *p)
{
	CPathEdge *e = next;
	while( e )
	{
		if( e->wall )
		{
			if( p ) *p = e->p1;
			return true;
		}

		e = e->next;
		if( e == this )
			break;
	}

	return false;
}

bool CPathEdge::FindWallCornerBefore(vector2 *p)
{
	CPathEdge *e = prev;
	while( e )
	{
		if( e->wall )
		{
			if( p ) *p = e->p2;
			return true;
		}

		e = e->prev;
		if( e == this )
			break;
	}

	return false;
}