#ifndef PATHMGR_H
#define PATHMGR_H

#include "acutil_math.h"
#include <vector>

class CPathNode;
class CPathEdge;

class CPathMgr
{
public:
	CPathMgr();
	~CPathMgr();

	// Build the map
	void SetArea(unsigned int width, unsigned int height);
	void SetWalkable(unsigned int x, unsigned int y, bool walkable);
	void Optimize();

	// Path finding
	acUtility::vector2 GetNearestPathNode(const acUtility::vector2 &from, const acUtility::vector2 &to);
	bool               IsWalkable(unsigned int x, unsigned int y);

	// Debugging
	void DebugDraw();

protected:
	float Heuristic(const acUtility::vector2 &from, const acUtility::vector2 &to);
	float Heuristic(unsigned int x1, unsigned int y1, unsigned int x2, unsigned int y2);
	float CostToNeighbour(unsigned int x1, unsigned int y1, unsigned int x2, unsigned int y2);

	acUtility::vector2 GetNearestUsingGrid(const acUtility::vector2 &from, const acUtility::vector2 &to);
	acUtility::vector2 GetNearestUsingNavMesh(const acUtility::vector2 &from, const acUtility::vector2 &to);

	void MakeRoom(const unsigned int x, const unsigned int y, bool *visited);

	char *area;
	unsigned int width;
	unsigned int height;

	std::vector<CPathNode*> nodes;
};

#endif