#include "pathnode.h"
#include "pathedge.h"

#include <vector>

using namespace std;
using namespace acUtility;

CPathNode::CPathNode(vector2 &v)
{
	p = v;
	first = 0;
}

CPathNode::~CPathNode()
{
	if( first )
	{
		CPathEdge *next = first->next;
		while( next && next != first )
		{
			CPathEdge *del = next;
			next = next->next;
			delete del;
		}
		delete first;
	}
}

bool CPathNode::IsPointInNode(const vector2 &v)
{
	// TODO: Speed up the check with a bounding circle
	CPathEdge *e = first;
	while( e )
	{
		// For the point to be in the node it has to be in front of all edges
		if( dot(v - e->p1, e->GetNormal()) < 0 )
			return false;

		e = e->next;
		if( e == first )
			break;
	}

	return true;
}

vector2 CPathNode::ShiftPathToAvoidWall(const vector2 &from, const vector2 &to)
{
	const float radius = 0.45f;

	vector2 path = to;

	CPathEdge *e = first;
	while( e )
	{
		// Check if the starting point is near an opening 
		float distToEdge1 = distancePointToLineSegment(from, e->p1, e->p2);
		if( !e->wall && distancePointToLineSegment(from, e->p1, e->p2) < radius )
		{
			// TODO: To be correct, we must also know if a corner is a wall, not just the edges
			vector2 p1, p2;
			if( e->FindWallCornerBefore(&p1) && e->FindWallCornerAfter(&p2) )
			{
				// Check if either corner is too close to the path with distancePointToLineSegment
				float distToPoint1 = distancePointToLineSegment(p1, from, to);
				if( distancePointToLineSegment(p1, from, to) < radius )
				{
					// If it is, project the corner onto the line with projectPointToLineSegment
					vector2 tmp = projectPointOntoLineSegment(p1, from, to);

					// Then push out the destination point from the corner with radius + 10%
					// The extra 10% is to avoid too many repeated adjustments when moving close to corners
					tmp = tmp - p1;
					tmp.normalize();
					p1 = p1 + tmp*(radius * 1.1f);

					float dist = (path - from).lengthSq();
					float dist1 = (p1 - from).lengthSq();
					// TODO: Make sure the adjustment isn't reversing the direction of the path
					if( dist1 < dist )
						path = p1;
				}
				
				float distToPoint2 = distancePointToLineSegment(p2, from, to);
				if( distancePointToLineSegment(p2, from, to) < radius )
				{
					// If it is, project the corner onto the line with projectPointToLineSegment
					vector2 tmp = projectPointOntoLineSegment(p2, from, to);

					// Then push out the destination point from the corner with radius + 10%
					// The extra 10% is to avoid too many repeated adjustments when moving close to corners
					tmp = tmp - p2;
					tmp.normalize();
					p2 = p2 + tmp*(radius * 1.1f);

					float dist = (path - from).lengthSq();
					float dist2 = (p2 - from).lengthSq();
					// TODO: Make sure the adjustment isn't reversing the direction of the path
					if( dist2 < dist )
						path = p2;
				}
			}
		}

		// Check if the end point is near an opening
		float distToEdge2 = distancePointToLineSegment(to, e->p1, e->p2);
		if( !e->wall && distancePointToLineSegment(to, e->p1, e->p2) < radius )
		{
			// TODO: To be correct, we must also know if a corner is a wall, not just the edges
			vector2 p1, p2;
			if( e->FindWallCornerBefore(&p1) && e->FindWallCornerAfter(&p2) )
			{
				// Check if either corner is too close to the path with distancePointToLineSegment
				float distToPoint1 = distancePointToLineSegment(p1, from, to);
				if( distancePointToLineSegment(p1, from, to) < radius )
				{
					// If it is, project the corner onto the line with projectPointToLineSegment
					vector2 tmp = projectPointOntoLineSegment(p1, from, to);

					// Then push out the destination point from the corner with radius + 10%
					// The extra 10% is to avoid too many repeated adjustments when moving close to corners
					tmp = tmp - p1;
					tmp.normalize();
					p1 = p1 + tmp*(radius * 1.1f);

					float dist = (path - from).lengthSq();
					float dist1 = (p1 - from).lengthSq();
					// TODO: Make sure the adjustment isn't reversing the direction of the path
					if( dist1 < dist )
						path = p1;
				}
				
				float distToPoint2 = distancePointToLineSegment(p2, from, to);
				if( distancePointToLineSegment(p2, from, to) < radius )
				{
					// If it is, project the corner onto the line with projectPointToLineSegment
					vector2 tmp = projectPointOntoLineSegment(p2, from, to);

					// Then push out the destination point from the corner with radius + 10%
					// The extra 10% is to avoid too many repeated adjustments when moving close to corners
					tmp = tmp - p2;
					tmp.normalize();
					p2 = p2 + tmp*(radius * 1.1f);

					float dist = (path - from).lengthSq();
					float dist2 = (p2 - from).lengthSq();
					// TODO: Make sure the adjustment isn't reversing the direction of the path
					if( dist2 < dist )
						path = p2;
				}
			}
		}

		e = e->next;
		if( e == first )
			break;
	}

	return path;
}

