#ifndef PHYSICSMGR_H
#define PHYSICSMGR_H

#include <vector>
#include "acutil_math.h"

class CPhysicsObject;
class b2World;
class b2Contact;
class b2Body;

enum EPhysicsProperty
{
	PP_SOLID,
	PP_PASSTHROUGH,
	PP_SENSOR
};

class CPhysicsMgr
{
public:
	CPhysicsMgr();
	~CPhysicsMgr();

	int  Init();
	void Uninit();
	void DebugDraw();
	void ClearWorld();

	void SetGlobalGravity(const acUtility::vector2 &vec);

	void Frame();
	int  GetCollisionCount();
	void GetCollision(int i, CPhysicsObject **a, CPhysicsObject **b, acUtility::vector2 *pos, acUtility::vector2 *norm, float *impulse);

	int  GetSensorProximityCount();
	void GetSensorProximity(int i, CPhysicsObject **a, CPhysicsObject **b, bool *isNearing);

	float RayCast(const CPhysicsObject *ignore, int query, const acUtility::vector2 &v1, const acUtility::vector2 &v2, CPhysicsObject **firstContact);
		
	bool CanObjectMoveTo(const acUtility::vector2 &pos, const CPhysicsObject *obj);

	CPhysicsObject *AddCollisionLine(const acUtility::vector2 &v1, const acUtility::vector2 &v2, bool isDynamic);
	CPhysicsObject *AddCollisionBox(const acUtility::vector2 &v1, const acUtility::vector2 &v2, bool isDynamic);

	CPhysicsObject *CreateObjectAsCircle(const acUtility::vector2 &pos, float collisionRadius, float weight, EPhysicsProperty prop);
	CPhysicsObject *CreateObjectAsBox(const acUtility::vector2 &pos, float halfWidth, float halfHeight, float weight, EPhysicsProperty prop);
	CPhysicsObject *CreateObjectAsCapsule(const acUtility::vector2 &pos, float halfWidth, float halfHeight, float weight, EPhysicsProperty prop);
	CPhysicsObject *CreateObjectAsBullet(const acUtility::vector2 &pos, float weight, EPhysicsProperty prop);
	void DestroyObject(CPhysicsObject *obj);

protected:
	friend class CContactListener;
	friend class CPhysicsObject;

	std::vector<CPhysicsObject*> objects;

	b2World *world;
	acUtility::vector2 gravity;

	struct SCollision
	{
		CPhysicsObject     *a;
		CPhysicsObject     *b;
		acUtility::vector2  pos;
		acUtility::vector2  norm;
		float               impulse;
	};

	struct SSensorProximity
	{
		CPhysicsObject     *sensor;
		CPhysicsObject     *object;
		bool                isNearing;
	};

	std::vector<SCollision>       collisions;
	std::vector<b2Body*>          ignoreContacts;
	std::vector<int>              ignoreCount;
	std::vector<SSensorProximity> sensorProximities;
	float nextFrame;
};

#endif