#include "physicsobject.h"
#include "gameobjectlink.h"
#include <box2d.h>
#include "application.h"
#include "physicsmgr.h"

using namespace acUtility;

CPhysicsObject::CPhysicsObject()
{
	m_link  = 0;
	m_group = 0;
	m_useCustomGravity = false;
}

CPhysicsObject::~CPhysicsObject()
{
	if( m_link )
		m_link->Release();
}

void CPhysicsObject::SetPosition(const acUtility::vector2 &pos)
{
	if( m_body )
		m_body->SetTransform(*reinterpret_cast<const b2Vec2*>(&pos), m_body->GetAngle());
}

vector2 CPhysicsObject::GetPosition() const
{
	if( m_body )
		return *reinterpret_cast<const vector2*>(&m_body->GetPosition());

	return vector2(0,0);
}

void CPhysicsObject::SetVelocity(const acUtility::vector2 &vel)
{
	if( m_body )
		m_body->SetLinearVelocity(*reinterpret_cast<const b2Vec2*>(&vel));
}

vector2 CPhysicsObject::GetVelocity() const
{
	if( m_body )
		return *reinterpret_cast<const vector2*>(&m_body->GetLinearVelocity());

	return vector2(0,0);
}

void CPhysicsObject::SetCollision(bool collision)
{
	if( m_body )
	{
		b2Fixture *fixture = m_body->GetFixtureList();
		while( fixture )
		{
			b2Filter filter = fixture->GetFilterData();
			filter.maskBits = collision ? 0xFFFF : 0;
			fixture->SetFilterData(filter);

			fixture = fixture->GetNext();
		}
	}
}

void CPhysicsObject::SetFixedRotation(bool fixed)
{
	if( m_body )
		m_body->SetFixedRotation(fixed);
}

void CPhysicsObject::SetCustomGravity(const acUtility::vector2 &gravity)
{
	m_gravity = gravity;
	m_useCustomGravity = true;
}

void CPhysicsObject::ClearCustomGravity()
{
	m_useCustomGravity = false;
}

float CPhysicsObject::GetRotation() const
{
	return m_body->GetAngle();
}

void CPhysicsObject::SetRotation(float rot)
{
	m_body->SetTransform(m_body->GetPosition(), rot);
}