#ifndef SCENEMGR_H
#define SCENEMGR_H

#include <vector>
#include <string>
#include "acutil_math.h"
#include "acgfx_graphics.h"
#include "sharedtexture.h"

namespace acUtility
{
	class CConfig;
}

class CSprite;
class CMaterial;


class CSceneMgr : public acGraphics::IHasDeviceObjects
{
public:
	CSceneMgr();
	~CSceneMgr();

	int  Init(acGraphics::CGraphics *gfx);
	void Uninit();
	void Render();
	void PrepareRender();

	void AddSprite(CSprite *sprite);
	void RemoveSprite(CSprite *sprite); // Remove a sprite without deleting it
	void RemoveAllSprites(); // Remove the sprites without deleting them
	void ClearScene(); // Delete all sprites

	int  GetMaterialIndex(const char *materialName);
	CMaterial *GetMaterialByIndex(int idx);

	void ReinitializeDeviceObjects(acGraphics::CGraphics *gfx);
	void ReleaseDeviceObjects();

	void SetCameraPosition(const acUtility::vector2 &pos);
	acUtility::vector2 GetCameraPosition();
	float GetCameraZoom();
	void SetCameraZoom(float zoom);

	acUtility::vector2 ConvertScreenToWorld(int x, int y);

	// Shared textures
	CSharedTexture *GetSharedTexture(const char *textureName);

	void SetFixedViewport(bool v);

	// Statistics
	void GetStats(int *numDraws, int *numMatChanges);

protected:
	void DetermineNearFarPlanes(float &znear, float &zfar);

	void RenderScene();

	int fxScene;

	acGraphics::CGraphics          *gfx;
	std::vector<CSprite*>           sprites;
	std::vector<CSprite*>           visibleSprites;
	std::vector<CMaterial*>         materials;
	std::vector<CSharedTexture*>    textures;

	acUtility::vector2              cameraPos;
	float                           cameraZoom;

	bool initialized;
	bool fixedViewport;

	// Statistics
	int numDrawCalls;
	int numMaterialChanges;
};

#endif