#include "scriptmathbinding.h"
#include "acutil_math.h"
#include <assert.h>
#include "angelscript.h"
#include <new>

using namespace acUtility;

void vector3_Constructor(void *ptr)
{
	new(ptr) vector3(0,0,0);
}

void vector3_Constructor(float x, float y, float z, void *ptr)
{
	new(ptr) vector3(x,y,z);
}

void vector2_Constructor(void *ptr)
{
	new(ptr) vector2(0,0);
}

void vector2_Constructor(float x, float y, void *ptr)
{
	new(ptr) vector2(x,y);
}

// The modf function doesn't seem very intuitive, so I'm writing this 
// function that simply returns the fractional part of the float value
float fraction(float v)
{
	float intPart;
	return modff(v, &intPart);
}

void RegisterScriptMath(asIScriptEngine *engine)
{
	int r;

	//-------------------------------
	// vector2
	//-------------------------------
	r = engine->RegisterObjectType("vector2", sizeof(vector2), asOBJ_VALUE | asOBJ_POD | asOBJ_APP_CLASS_CA); assert( r >= 0 );
	r = engine->RegisterObjectBehaviour("vector2", asBEHAVE_CONSTRUCT, "void f()", asFUNCTIONPR(vector2_Constructor, (void*), void), asCALL_CDECL_OBJLAST); assert( r >= 0 );
	r = engine->RegisterObjectBehaviour("vector2", asBEHAVE_CONSTRUCT, "void f(float, float)", asFUNCTIONPR(vector2_Constructor, (float, float, void*), void), asCALL_CDECL_OBJLAST); assert( r >= 0 );
	r = engine->RegisterObjectMethod("vector2", "vector2 &opAssign(const vector2 &in)", asMETHODPR(vector2, operator=, (const vector2 &), vector2 &), asCALL_THISCALL); assert( r >= 0 ); 
	r = engine->RegisterObjectMethod("vector2", "vector2 &opAddAssign(const vector2 &in)", asMETHODPR(vector2, operator+=, (const vector2 &), vector2 &), asCALL_THISCALL); assert( r >= 0 );
	r = engine->RegisterObjectMethod("vector2", "vector2 &opSubAssign(const vector2 &in)", asMETHODPR(vector2, operator-=, (const vector2 &), vector2 &), asCALL_THISCALL); assert( r >= 0 );
	r = engine->RegisterObjectMethod("vector2", "vector2 &opMulAssign(float)", asMETHODPR(vector2, operator*=, (float), vector2 &), asCALL_THISCALL); assert( r >= 0 );
	r = engine->RegisterObjectMethod("vector2", "vector2 &opDivAssign(float)", asMETHODPR(vector2, operator/=, (float), vector2 &), asCALL_THISCALL); assert( r >= 0 );
	r = engine->RegisterObjectMethod("vector2", "bool opEquals(const vector2 &in) const", asMETHODPR(vector2, operator==, (const vector2 &) const, bool), asCALL_THISCALL); assert( r >= 0 ); 
	r = engine->RegisterObjectMethod("vector2", "vector2 opAdd(const vector2 &in) const", asMETHODPR(vector2, operator+, (const vector2 &) const, vector2), asCALL_THISCALL); assert( r >= 0 ); 
	r = engine->RegisterObjectMethod("vector2", "vector2 opSub(const vector2 &in) const", asMETHODPR(vector2, operator-, (const vector2 &) const, vector2), asCALL_THISCALL); assert( r >= 0 ); 
	r = engine->RegisterObjectMethod("vector2", "vector2 opMul(float) const", asMETHODPR(vector2, operator*, (float) const, vector2), asCALL_THISCALL); assert( r >= 0 ); 
	r = engine->RegisterObjectMethod("vector2", "vector2 opDiv(float) const", asMETHODPR(vector2, operator/, (float) const, vector2), asCALL_THISCALL); assert( r >= 0 ); 
	r = engine->RegisterObjectMethod("vector2", "vector2 opNeg() const", asMETHODPR(vector2, operator-, () const, vector2), asCALL_THISCALL); assert( r >= 0 );
	r = engine->RegisterObjectMethod("vector2", "vector2 opMul_r(float) const", asFUNCTIONPR(operator*,(float, const vector2 &), vector2), asCALL_CDECL_OBJLAST); assert( r >= 0 );
	r = engine->RegisterObjectMethod("vector2", "float length() const", asMETHOD(vector2, length), asCALL_THISCALL); assert( r >= 0 );
	r = engine->RegisterObjectMethod("vector2", "float lengthSq() const", asMETHOD(vector2, lengthSq), asCALL_THISCALL); assert( r >= 0 );

	r = engine->RegisterObjectProperty("vector2", "float x", asOFFSET(vector2, x)); assert( r >= 0 );
	r = engine->RegisterObjectProperty("vector2", "float y", asOFFSET(vector2, y)); assert( r >= 0 );

	r = engine->RegisterGlobalFunction("float dot(const vector2 &in, const vector2 &in)", asFUNCTIONPR(dot, (const vector2 &, const vector2 &), float), asCALL_CDECL); assert( r >= 0 );

	//-------------------------------
	// vector3
	//-------------------------------
	r = engine->RegisterObjectType("vector3", sizeof(vector3), asOBJ_VALUE | asOBJ_POD | asOBJ_APP_CLASS_CA); assert( r >= 0 );
	r = engine->RegisterObjectBehaviour("vector3", asBEHAVE_CONSTRUCT, "void f()", asFUNCTIONPR(vector3_Constructor, (void*), void), asCALL_CDECL_OBJLAST); assert( r >= 0 );
	r = engine->RegisterObjectBehaviour("vector3", asBEHAVE_CONSTRUCT, "void f(float, float, float)", asFUNCTIONPR(vector3_Constructor, (float, float, float, void*), void), asCALL_CDECL_OBJLAST); assert( r >= 0 );
	r = engine->RegisterObjectMethod("vector3", "vector3 &opAssign(const vector3 &in)", asMETHODPR(vector3, operator=, (const vector3 &), vector3 &), asCALL_THISCALL); assert( r >= 0 ); 
	r = engine->RegisterObjectMethod("vector3", "vector3 &opAddAssign(const vector3 &in)", asMETHODPR(vector3, operator+=, (const vector3 &), vector3 &), asCALL_THISCALL); assert( r >= 0 );
	r = engine->RegisterObjectMethod("vector3", "vector3 &opSubAssign(const vector3 &in)", asMETHODPR(vector3, operator-=, (const vector3 &), vector3 &), asCALL_THISCALL); assert( r >= 0 );
	r = engine->RegisterObjectMethod("vector3", "vector3 &opMulAssign(float)", asMETHODPR(vector3, operator*=, (float), vector3 &), asCALL_THISCALL); assert( r >= 0 );
	r = engine->RegisterObjectMethod("vector3", "vector3 &opDivAssign(float)", asMETHODPR(vector3, operator/=, (float), vector3 &), asCALL_THISCALL); assert( r >= 0 );
	r = engine->RegisterObjectMethod("vector3", "bool opEquals(const vector3 &in) const", asMETHODPR(vector3, operator==, (const vector3 &) const, bool), asCALL_THISCALL); assert( r >= 0 ); 
	r = engine->RegisterObjectMethod("vector3", "vector3 opAdd(const vector3 &in) const", asMETHODPR(vector3, operator+, (const vector3 &) const, vector3), asCALL_THISCALL); assert( r >= 0 ); 
	r = engine->RegisterObjectMethod("vector3", "vector3 opSub(const vector3 &in) const", asMETHODPR(vector3, operator-, (const vector3 &) const, vector3), asCALL_THISCALL); assert( r >= 0 ); 
	r = engine->RegisterObjectMethod("vector3", "vector3 opMul(float) const", asMETHODPR(vector3, operator*, (float) const, vector3), asCALL_THISCALL); assert( r >= 0 ); 
	r = engine->RegisterObjectMethod("vector3", "vector3 opDiv(float) const", asMETHODPR(vector3, operator/, (float) const, vector3), asCALL_THISCALL); assert( r >= 0 ); 
	r = engine->RegisterObjectMethod("vector3", "vector3 opNeg() const", asMETHODPR(vector3, operator-, () const, vector3), asCALL_THISCALL); assert( r >= 0 );
	r = engine->RegisterObjectMethod("vector3", "vector3 opMul_r(float) const", asFUNCTIONPR(operator*,(float, const vector3 &), vector3), asCALL_CDECL_OBJLAST); assert( r >= 0 );
	r = engine->RegisterObjectMethod("vector3", "float length() const", asMETHOD(vector3, length), asCALL_THISCALL); assert( r >= 0 );
	r = engine->RegisterObjectMethod("vector3", "float lengthSq() const", asMETHOD(vector3, lengthSq), asCALL_THISCALL); assert( r >= 0 );

	r = engine->RegisterObjectProperty("vector3", "float x", asOFFSET(vector3, x)); assert( r >= 0 );
	r = engine->RegisterObjectProperty("vector3", "float y", asOFFSET(vector3, y)); assert( r >= 0 );
	r = engine->RegisterObjectProperty("vector3", "float z", asOFFSET(vector3, z)); assert( r >= 0 );

	r = engine->RegisterGlobalFunction("float dot(const vector3 &in, const vector3 &in)", asFUNCTIONPR(dot, (const vector3 &, const vector3 &), float), asCALL_CDECL); assert( r >= 0 );
	r = engine->RegisterGlobalFunction("vector3 cross(const vector3 &in, const vector3 &in)", asFUNCTIONPR(cross, (const vector3 &, const vector3 &), vector3), asCALL_CDECL); assert( r >= 0 );

	//--------------------------
	// Scalar math
	//--------------------------

	// Trigonometric functions
	r = engine->RegisterGlobalFunction("float cos(float)", asFUNCTION(cosf), asCALL_CDECL); assert( r >= 0 );
	r = engine->RegisterGlobalFunction("float sin(float)", asFUNCTION(sinf), asCALL_CDECL); assert( r >= 0 );
	r = engine->RegisterGlobalFunction("float tan(float)", asFUNCTION(tanf), asCALL_CDECL); assert( r >= 0 );

	r = engine->RegisterGlobalFunction("float acos(float)", asFUNCTION(acosf), asCALL_CDECL); assert( r >= 0 );
	r = engine->RegisterGlobalFunction("float asin(float)", asFUNCTION(asinf), asCALL_CDECL); assert( r >= 0 );
	r = engine->RegisterGlobalFunction("float atan(float)", asFUNCTION(atanf), asCALL_CDECL); assert( r >= 0 );
	r = engine->RegisterGlobalFunction("float atan(float y, float x)", asFUNCTIONPR(atan2, (float, float), float), asCALL_CDECL); assert( r >= 0 );

	// Hyberbolic functions
	r = engine->RegisterGlobalFunction("float cosh(float)", asFUNCTION(coshf), asCALL_CDECL); assert( r >= 0 );
	r = engine->RegisterGlobalFunction("float sinh(float)", asFUNCTION(sinhf), asCALL_CDECL); assert( r >= 0 );
	r = engine->RegisterGlobalFunction("float tanh(float)", asFUNCTION(tanhf), asCALL_CDECL); assert( r >= 0 );

	// Exponential and logarithmic functions
	r = engine->RegisterGlobalFunction("float log(float)", asFUNCTION(logf), asCALL_CDECL); assert( r >= 0 );
	r = engine->RegisterGlobalFunction("float log10(float)", asFUNCTION(log10f), asCALL_CDECL); assert( r >= 0 );

	// Power functions
	r = engine->RegisterGlobalFunction("float pow(float, float)", asFUNCTION(powf), asCALL_CDECL); assert( r >= 0 );
	r = engine->RegisterGlobalFunction("float sqrt(float)", asFUNCTION(sqrtf), asCALL_CDECL); assert( r >= 0 );

	// Nearest integer, absolute value, and remainder functions
	r = engine->RegisterGlobalFunction("float ceil(float)", asFUNCTION(ceilf), asCALL_CDECL); assert( r >= 0 );
	r = engine->RegisterGlobalFunction("float abs(float)", asFUNCTION(fabsf), asCALL_CDECL); assert( r >= 0 );
	r = engine->RegisterGlobalFunction("float floor(float)", asFUNCTION(floorf), asCALL_CDECL); assert( r >= 0 );
	r = engine->RegisterGlobalFunction("float fraction(float)", asFUNCTION(fraction), asCALL_CDECL); assert( r >= 0 );

	// Don't register modf because AngelScript already supports the % operator
}

